@@ -36,7 +36,7 @@ Contributors:
 #endif
 
 #if !defined(WIN32) && !defined(__CYGWIN__)
-#  include <sys/syslog.h>
+#  include <syslog.h>
 #endif
 
 #include <mosquitto_broker.h>
@@ -309,7 +309,7 @@ void mqtt3_config_cleanup(struct mqtt3_config *config)
 static void print_usage(void)
 {
 	printf("mosquitto version %s (build date %s)\n\n", VERSION, TIMESTAMP);
-	printf("mosquitto is an MQTT v3.1 broker.\n\n");
+	printf("mosquitto is an MQTT v3.1.1/v3.1 broker.\n\n");
 	printf("Usage: mosquitto [-c config_file] [-d] [-h] [-p port]\n\n");
 	printf(" -c : specify the broker config file.\n");
 	printf(" -d : put the broker into the background after starting.\n");
